﻿using Infragistics.Win.DataVisualization;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Runtime.CompilerServices;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace UltraPieChartCheckboxItemDemo
{
    public partial class Form1 : Form
    {

        public Data MasterDataSource { get; set; }
        public Data PieDataSource { get; set; }

        public BrushCollection MasterBrushes { get; set; }
        public BrushCollection PieBrushes { get; set; }


        System.Drawing.Brush[] brushes = new System.Drawing.Brush[] { Brushes.Red, Brushes.Orange, Brushes.Yellow, Brushes.Green, Brushes.Blue };

        public Form1()
        {
            InitializeComponent();

            MasterDataSource = new Data();
            PieDataSource = new Data();

            MasterBrushes = new BrushCollection();
            PieBrushes = new BrushCollection();
            foreach (System.Drawing.Brush brush in brushes)
            {
                MasterBrushes.Add(brush);
                PieBrushes.Add(brush);
            }

            ultraPieChart1.Outlines = new BrushCollection() { Brushes.Black };
            ultraPieChart2.Outlines = new BrushCollection() { Brushes.Black };

            for (int i=0; i < PieDataSource.Count; i++)
            {
                PieDataSource[i].AssignedBrush = PieBrushes[i] as SolidColorBrush;                
            }

            ultraPieChart1.DataSource = MasterDataSource;
            ultraPieChart1.LabelMemberPath = "Label";
            ultraPieChart1.ValueMemberPath = "Value";
            ultraPieChart1.Brushes = MasterBrushes;

            ultraPieChart1.LegendLabelMemberPath = "Label";
            ultraPieChart1.Legend = ultraItemLegend1;

            ultraPieChart2.DataSource = PieDataSource;
            ultraPieChart2.LabelMemberPath = "Label";
            ultraPieChart2.ValueMemberPath = "Value";
            ultraPieChart2.Brushes = PieBrushes;

            this.Load += Form1_Load;
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            var x = ultraItemLegend1.Controls;

            Control flpe = ultraItemLegend1.Controls[0] as Control;

            foreach(FlowLayoutPanel flp in flpe.Controls)
            {
                CheckBox check = new CheckBox() { Width = 16, Checked = true };
                check.CheckedChanged += Check_CheckedChanged;
                flp.Controls.Add(check);                
                flp.Controls.SetChildIndex(check, 0);                
            }
        }
        
        private void Check_CheckedChanged(object sender, EventArgs e)
        {
            CheckBox check = sender as CheckBox;
            FlowLayoutPanel flp = check.Parent as FlowLayoutPanel;

            Label lbl = flp.Controls[flp.Controls.Count - 1] as Label;
            string legendText = lbl.Text;

            DataItem pieItem = PieDataSource.Where(i => i.Label == legendText).FirstOrDefault();

            int index = PieDataSource.IndexOf(pieItem);
            int insertIndex = index;

            if (check.Checked)
            {                
                DataItem masterItem = MasterDataSource.Where(i => i.Label == legendText).FirstOrDefault();
                pieItem.Value = masterItem.Value;       
                
                if(index > PieBrushes.Count)
                {
                    insertIndex = PieBrushes.Count;
                }

                PieBrushes.Insert(insertIndex, MasterBrushes[index]);                
            }
            else
            {
                pieItem.Value = 0;
                if (PieBrushes.Contains(pieItem.AssignedBrush))
                {
                    PieBrushes.Remove(pieItem.AssignedBrush);
                }
            }
        }
    }

    public class DataItem : INotifyPropertyChanged
    {
        private string _label;
        private double _value;
        private SolidColorBrush _brush;

        public string Label
        {
            get { return _label; }
            set
            {
                _label = value;
                NotifyPropertyChanged();
            }
        }

        public double Value
        {
            get { return _value; }
            set
            {
                _value = value;
                NotifyPropertyChanged();
            }
        }

        public SolidColorBrush AssignedBrush
        {
            get { return _brush; }
            set
            {
                _brush = value;
                NotifyPropertyChanged();
            }

        }


        public event PropertyChangedEventHandler PropertyChanged;
        private void NotifyPropertyChanged([CallerMemberName] String propertyName = "")
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }


    }
    public class Data : ObservableCollection<DataItem>
    {
        public Data()
        {
            Add(new DataItem { Label = "Item 1", Value = 5 });
            Add(new DataItem { Label = "Item 2", Value = 6 });
            Add(new DataItem { Label = "Item 3", Value = 3 });
            Add(new DataItem { Label = "Item 4", Value = 7 });
            Add(new DataItem { Label = "Item 5", Value = 5 });
        }
    }
}
